
/*

-Procedure turnpro ( Generate a turn orientation profile time series. )

   Jeff Bytof, last update 14-May-2001

*/

#include "SpiceUsr.h"
#include "SpiceZmc.h"
#include "SpiceZfc.h"

void turnpro ( SpiceDouble      t, 
               SpiceDouble      tb, 
               SpiceDouble      tc, 
               SpiceDouble      angacc    [3], 

               SpiceInt         nb,
               SpiceInt         nc,
               SpiceInt         ROOM,
               SpiceInt       * nout,

               SpiceDouble      trntim    [ROOM],
               SpiceDouble      trnrot    [ROOM][3][3]
             )

{


/*

-Brief_I/O

    Variable    I/O      Description
    --------    ---      -----------
    t            I       Total duration of turn in seconds.
    tb           I       Duration of spinup or spindown in seconds.
    tc           I       Duration of coast in seconds.
    angacc       I       Constant angular acceleration vector at spinup.
    nb           I       Number of points returned for spinup/down phases.
    nc           I       Number of points returned for coast phase.
    ROOM         I       Dimension of arrays in calling program.

    nout         O       Number of returned orientations.
    trntim       O       Orientation time array, in seconds.
    trnrot       O       Orientation array.

*/
   
/*
   Local variables.
*/

   SpiceDouble   av         [3];
   SpiceDouble   dtb; 
   SpiceDouble   dtc; 
   SpiceDouble   infrot     [3][3]; 
   SpiceDouble   rot        [3][3];
   SpiceDouble   tbase; 
 
   SpiceInt      i;
   SpiceInt      i1;
   SpiceInt      i2;
   SpiceInt      i3;
   SpiceInt      j;
   SpiceInt      k;
   SpiceInt      ncount; 


/*
   Initialize orientation matrix and angular velocity.
*/
 
   ident_c ( rot ); 

   i1 = 1;
   i2 = 2;
   i3 = 3;

   for ( i=0; i<3; i++ ) {
      av[i] = 0.0;
   }

/*
   Profile spinup orientations.
*/

   dtb = tb / ((double) (nb-1));

   ncount = 0; 

   tbase         = 0.0;
   trntim[ncount] = tbase;

   for ( j=0; j<3; j++ ) {
      for ( k=0; k<3; k++ ) { 
         trnrot[ncount][j][k] = rot[j][k];
      }
   }

   for ( i=0; i<(nb-1); i++ ) { 
      
      for ( j=0; j<3; j++ ) { 
         av[j] = av[j] + angacc[j]*dtb;
      }
 
      eul2m_c ( av[2]*dtb, av[1]*dtb, av[0]*dtb,
                       i3,        i2,        i1,  infrot );

/*
   Apply infinitesimal rotation.
*/

      mxm_c ( infrot, rot, rot );

      ncount++;
 
      if ( ncount > ROOM ) { 
         *nout = -ncount;
         return;
      }

      trntim[ncount] = dtb * (double) i;

      for ( j=0; j<3; j++ ) { 
         for ( k=0; k<3; k++ ) { 
            trnrot[ncount][j][k] = rot[j][k];
         }
      } 
   } 

/*
   Profile coasting orientations.
*/

   dtc = tc / ((double) (nc-1));

   tbase = tb;

   for ( i=0; i<(nc-1); i++ ) {
      
      eul2m_c ( av[2]*dtc, av[1]*dtc, av[0]*dtc,
                       i3,        i2,        i1,   infrot );

/* 
   apply infinitesimal rotation.
*/

      mxm_c ( infrot, rot, rot );

      ncount++; 

      if ( ncount > ROOM ) { 
         *nout = -ncount;
         return;
      }

      trntim[ncount] = tbase + dtc * (double) i; 

      for ( j=0; j<3; j++ ) { 
         for ( k=0; k<3; k++ ) { 
            trnrot[ncount][j][k] = rot[j][k];
         }
      }
   }

   
/*
   Profile spindown orientations.
*/

   dtb = tb / ((double) (nb-1));

   tbase = tb + tc;

   
/*
   Apply reverse angular acceleration.
*/

   for ( i=0; i<(nb-1); i++ ) { 

      for ( j=0; j<3; j++ ) { 
         av[j] = av[j] - angacc[j]*dtb;
      }

      eul2m_c ( av[2]*dtb, av[1]*dtb, av[0]*dtb, 
                       i3,        i2,        i1,  infrot );

/* 
   Apply infinitesimal rotation.
*/

      mxm_c ( infrot, rot, rot );

      ncount++; 

      if ( ncount > ROOM ) { 
         *nout = -ncount;
         return;
      }

      trntim[ncount] = tbase + dtb * (double) i;

      for ( j=0; j<3; j++ ) { 
         for ( k=0; k<3; k++ ) { 
            trnrot[ncount][j][k] = rot[j][k];
         } 
      } 
   }

   *nout = ncount;

   return; 
}
